interface DateFormat {
  /** return: 2024/02/27 20:28:49
       * 
      return: 1402/12/08 20:28:49 */
  YYYY_MM_dd_HH_MM_ss: string;
  /** return: 2024/02/27 20:28
       * 
      return: 1402/12/08 20:28 */
  YYYY_MM_dd_HH_MM: string;
  /** return: 20:29:59 */
  HH_MM_ss: string;
  /** return: 20:29 */
  HH_MM: string;
  /** return: 2024/02/27
       * 
      return: 1402/12/08 */
  YYYY_MM_dd: string;
  /** return: 27 February 2024
       * 
      return: 08 اسفند 1402 */
  YYYY_M_dd: string;
  /** return: February 2024
       * 
      return: اسفند 1402 */
  YYYY_M: string;
  /** return: Tuesday 27 February 2024
       * 
      return: سه‌شنبه 08 اسفند 1402 */
  W_YYYY_M_dd: string;
  /** return: Tuesday 27 February
      * 
      return: سه‌شنبه 8 اسفند  */
  W_dd_M: string;
  /** return: 27 February
      * 
      return: 8 اسفند  */
  dd_M: string;
  /** return: Year */
  YYYY: string;
  /** return: Month Number two digit */
  MM: string;
  /** return: Month name */
  M: string;
  /** return: day of month */
  d: string;
  /** return: Week name */
  W: string;
  /** return: Weekday number */
  WW: string;
  /** return: Unix */
  Unix: number;
}

function format(
  locale: string,
  value: string | Date,
  separator?: string
): DateFormat {
  if (!separator) separator = "/";

  const date = new Date(String(value));
  const monthName = new Intl.DateTimeFormat(locale, {
    month: "long",
  }).format(date);

  const fmt: Record<string, string> = new Intl.DateTimeFormat(locale, {
    weekday: "long",
    day: "numeric",
    month: "numeric",
    year: "numeric",
    hour: "numeric",
    minute: "numeric",
    second: "numeric",
    hour12: false,
  })
    .formatToParts(date)
    .reduce((acc: Record<string, string>, part) => {
      acc[part.type] = toEnglishNumber(String(part.value));
      return acc;
    }, {});

  fmt.month = fmt.month.toString().padStart(2, "0");
  fmt.day = fmt.day.toString().padStart(2, "0");

  return {
    YYYY_MM_dd_HH_MM_ss: `${
      fmt.year + separator + fmt.month + separator + fmt.day
    } ${fmt.hour}:${fmt.minute}:${fmt.second}`,
    YYYY_MM_dd_HH_MM: `${
      fmt.year + separator + fmt.month + separator + fmt.day
    } ${fmt.hour}:${fmt.minute}`,
    HH_MM_ss: `${fmt.hour}:${fmt.minute}:${fmt.second}`,
    HH_MM: `${fmt.hour}:${fmt.minute}`,
    YYYY_MM_dd: `${fmt.year + separator + fmt.month + separator + fmt.day}`,
    YYYY_M_dd: ` ${fmt.day} ${monthName} ${fmt.year}`,
    YYYY_M: `${fmt.year} ${monthName}`,
    W_YYYY_M_dd: `${fmt.weekday} ${fmt.day} ${monthName} ${fmt.year}`,
    W_dd_M: `${fmt.weekday} ${fmt.day} ${monthName}`,
    dd_M: `${fmt.day} ${monthName}`,
    YYYY: `${fmt.year}`,
    MM: fmt.month,
    M: monthName,
    d: fmt.day,
    W: `${fmt.weekday}`,
    WW: getWeekdayNumber(fmt.weekday, locale).toString(),
    Unix: Math.floor(new Date(date).getTime() / 1000),
  };
}

function convert(
  locale: string,
  value: string | Date,
  separator?: string
): DateFormat {
  if (locale.toLocaleLowerCase() === "fa") {
    return format("fa", value, separator);
  } else {
    // Assuming toGregorian is implemented elsewhere
    return toGregorian(value, separator);
  }
}

function addDays(
  locale: string,
  value: string,
  days: number,
  separator?: string
): DateFormat {
  if (locale.toLocaleLowerCase() === "fa") {
    const date = new Date(convert("en", value)?.YYYY_MM_dd);
    date.setDate(date.getDate() + days);
    return format("fa", date, separator);
  } else {
    const date = new Date(String(value));
    date.setDate(date.getDate() + days);
    return format("en", date, separator);
  }
}

function DaysInMonth(locale: string, year: number, month: number): number {
  if (locale.toLowerCase() === "fa") {
    const date1: Date = new Date(
      convert("en", `${year}/${month}/1`)?.YYYY_MM_dd
    );
    if (month === 12) {
      year++;
      month = 1;
    } else {
      month++;
    }
    const date2: Date = new Date(
      convert("en", `${year}/${month}/1`)?.YYYY_MM_dd
    );
    return Math.round(
      (date2.getTime() - date1.getTime()) / (1000 * 60 * 60 * 24)
    );
  } else {
    const d: Date = new Date(year, month, 0);
    return d.getDate();
  }
}

function datoToNumber(date: Date | string): number {
  return Number(
    toEnglishNumber(String(date).replace(/\//g, "").replace(/-/g, ""))
  );
}

//#region private functions
function toEnglishNumber(value: string): string {
  if (value == null) return "";
  const englishNumbers = "0123456789";
  const persianNumbers1 = "٠١٢٣۴۵۶٧٨٩";
  const persianNumbers2 = "۰۱۲۳۴۵۶۷۸۹";
  const arabicNumbers = "٠١٢٣٤٥٦٧٨٩";
  let res = "";
  for (const c of value) {
    const index1 = persianNumbers1.indexOf(c);
    const index2 = persianNumbers2.indexOf(c);
    const index3 = arabicNumbers.indexOf(c);
    const index = index1 >= 0 ? index1 : index2 >= 0 ? index2 : index3;
    res += index >= 0 ? englishNumbers[index] : c;
  }

  return res;
}

function toGregorian(value: string | Date, separator?: string): DateFormat {
  // Implementation of toGregorian is not provided, please add it if necessary
  // Assuming toGregorian is implemented elsewhere
  const JalaliDate = {
    g_days_in_month: [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31],
    j_days_in_month: [31, 31, 31, 31, 31, 31, 30, 30, 30, 30, 30, 29],
  };

  if (!separator) separator = "/";

  const literal = String(value).indexOf("-") > 0 ? "-" : "/";

  const j_y: number = +toEnglishNumber(String(value).split(literal)[0]);
  const j_m: number = +toEnglishNumber(String(value).split(literal)[1]);
  const j_d: number = +toEnglishNumber(String(value).split(literal)[2]);

  if (value != null) {
    const jy: number = j_y - 979;
    const jm: number = j_m - 1;
    const jd: number = j_d - 1;

    let j_day_no: number =
      365 * jy + Math.floor(jy / 33) * 8 + Math.floor(((jy % 33) + 3) / 4);
    for (let i: number = 0; i < jm; ++i) {
      j_day_no += JalaliDate.j_days_in_month[i];
    }

    j_day_no += jd;

    let g_day_no = j_day_no + 79;

    let gy: number =
      1600 +
      400 *
        Math.floor(
          g_day_no / 146097
        ); /* 146097 = 365*400 + 400/4 - 400/100 + 400/400 */
    g_day_no = g_day_no % 146097;

    let leap: boolean = true;
    if (g_day_no >= 36525) {
      /* 36525 = 365*100 + 100/4 */
      g_day_no--;
      gy +=
        100 *
        Math.floor(g_day_no / 36524); /* 36524 = 365*100 + 100/4 - 100/100 */
      g_day_no = g_day_no % 36524;

      if (g_day_no >= 365) g_day_no++;
      else leap = false;
    }

    gy += 4 * Math.floor(g_day_no / 1461); /* 1461 = 365*4 + 4/4 */
    g_day_no %= 1461;

    if (g_day_no >= 366) {
      leap = false;

      g_day_no--;
      gy += Math.floor(g_day_no / 365);
      g_day_no = g_day_no % 365;
    }

    // let i: number;
    // for (i = 0; g_day_no >= JalaliDate.g_days_in_month[i] + (i === 1 && leap); i++) {
    //   g_day_no -= JalaliDate.g_days_in_month[i] + (i === 1 && leap);
    // }

    let i: number;
    for (
      i = 0;
      g_day_no >= JalaliDate.g_days_in_month[i] + (i === 1 && leap ? 1 : 0);
      i++
    ) {
      g_day_no -= JalaliDate.g_days_in_month[i] + (i === 1 && leap ? 1 : 0);
    }

    let gm: number = i + 1;
    let gd: number = g_day_no + 1;

    gm = gm < 10 ? 0 + gm : gm;
    gd = gd < 10 ? 0 + gd : gd;

    return format(
      "en",
      new Date(`${gy + separator + gm + separator + gd}`),
      separator
    );
  } else {
    return format("en", new Date(), separator);
  }
}

function getWeekdayNumber(weekdayString: string, locale: string): number {
  if (locale === "en") {
    return [
      "Saturday",
      "Sunday",
      "Monday",
      "Tuesday",
      "Wednesday",
      "Thursday",
      "Friday",
    ].indexOf(weekdayString);
  } else {
    return [
      "شنبه",
      "یکشنبه",
      "دوشنبه",
      "سه‌شنبه",
      "چهارشنبه",
      "پنجشنبه",
      "جمعه",
    ].indexOf(weekdayString);
  }
}
//#endregion private functions

const sdate = {
  format,
  convert,
  addDays,
  DaysInMonth,
  datoToNumber,
};

export default sdate;
